==> osso-statusbar-cpu_0.4.1-1.tar.gz: osso-statusbar-cpu-0.4.1/src/osso-statusbar-cpu.c <==
#include <hildon-status-bar-lib/hildon-status-bar-item.h>
#include <stdlib.h>
#include <signal.h>

/* GTK includes */
#include <gtk/gtkicontheme.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtkmenu.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkseparatormenuitem.h>
#include <hildon-widgets/hildon-note.h>

#include <glib.h>

#include "dialog.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include "osso-statusbar-cpu.h"
#include "common.h"

static gboolean my_timeout(gpointer data);
static void my_gconf_handler(GConfClient * gcc,
			     guint conn,
			     GConfEntry * entry, gpointer user_data);
static void cpu_inact(osso_hw_state_t * state, gpointer user_data);
static void cpu_popup(GtkToggleButton *button, gpointer user_data);
static void my_notify(gpointer data, GObject *has_been);

void cpu_entry(HildonStatusBarPluginFn_st *fn)
{
	g_return_if_fail(fn);
	fn->initialize = cpu_initialize;
	fn->destroy = cpu_destroy;
	fn->update = cpu_update;
	fn->get_priority = cpu_get_priority;
}

void *cpu_initialize(HildonStatusBarItem *item, GtkWidget **button)
{
	osso_hw_state_t osso_states = {
		.shutdown_ind = FALSE,
		.save_unsaved_data_ind = FALSE,
		.memory_low_ind = FALSE,
		.system_inactivity_ind = TRUE,
		.sig_device_mode_ind = 0
       	};
	GdkPixbuf *icon_pixbuf = NULL;
	GError *error = NULL;
	PluginInfo *info = NULL;
	GtkIconTheme *theme = NULL;
	g_return_val_if_fail(item, NULL);

	theme = gtk_icon_theme_get_default();
	icon_pixbuf = gtk_icon_theme_load_icon(theme, HILDON_STATUS_BAR_CPU_ICON_NAME,
			40, GTK_ICON_LOOKUP_NO_SVG, NULL);
	if (icon_pixbuf == NULL) {
		gtk_widget_destroy(GTK_WIDGET(item));
		return NULL;
	}

	info = g_new0(PluginInfo, 1);
	g_return_val_if_fail(info, NULL);

	info->item = item;

	info->osso =
		osso_initialize("osso_statusbar_cpu", VERSION, FALSE,
				NULL);
	osso_hw_set_event_cb(info->osso, &osso_states, cpu_inact,
			     (gpointer)info);

	info->gcc = gconf_client_get_default();

	info->button = gtk_toggle_button_new();

	if (error) {
		g_error_free(error);
		error = NULL;
		info->icon = NULL;
	}
	info->icon = gtk_image_new_from_pixbuf(icon_pixbuf);

	info->color = gc_get_int_fall(info->gcc,
				      GCONF_PATH_CPU_COLOR, CPU_DEFAULT);

	/* icon_pixbuf is owned by info->icon */
	g_object_unref(icon_pixbuf);
	
	gtk_container_add(GTK_CONTAINER(info->button),
			  GTK_WIDGET(info->icon));

	*button = info->button;

	info->notify_id = g_timeout_add(2000, my_timeout, info);

	gconf_client_add_dir(info->gcc,
			     GCONF_PATH, GCONF_CLIENT_PRELOAD_NONE, NULL);

	info->gc_notify_id = gconf_client_notify_add(info->gcc,
						     GCONF_PATH_CPU_COLOR,
						     my_gconf_handler,
						     info, NULL, NULL);

	g_signal_connect(info->button, "toggled", G_CALLBACK(cpu_popup),
			 info);

	gconf_client_clear_cache(info->gcc);
	if (!gconf_client_get_bool(info->gcc, GCONF_PATH_CPU_ON, NULL)) {
		gtk_object_destroy(GTK_OBJECT(item));
	} else {
		gtk_widget_show_all(info->button);
	}

	return info;
}

void cpu_destroy(void *data)
{
	osso_hw_state_t osso_states = { FALSE, FALSE, FALSE, FALSE, 0 };
	PluginInfo *info;
	g_return_if_fail(data);

	info = (PluginInfo *)data;

	g_source_remove(info->notify_id);
	info->notify_id = 0;

	gconf_client_notify_remove(info->gcc, info->gc_notify_id);
	gconf_client_remove_dir(info->gcc, GCONF_PATH, NULL);

	g_object_unref(G_OBJECT(info->gcc));

	osso_hw_unset_event_cb(info->osso, &osso_states);
	osso_deinitialize(info->osso);

	g_free(data);
}

gint cpu_get_priority(void *data)
{
	return HILDON_STATUS_BAR_CPU_PRIORITY;
}

void cpu_update(void *data, gint value1, gint value2, const gchar *str)
{
	PluginInfo *info = (PluginInfo *)data;
	g_return_if_fail(info);

	if (value2 == 0)
		gtk_widget_destroy(GTK_WIDGET(info->item));
	return;
}

static gboolean my_timeout(gpointer data)
{
	PluginInfo *info = (PluginInfo *)data;
	GtkImage *image = GTK_IMAGE(info->icon);
	GdkPixbuf *buf = gtk_image_get_pixbuf(image);

	FILE *foo = fopen("/proc/stat", "r");
	int user, nice, kern, idle, total_change;
	static int old_user, old_nice, old_kern, old_idle;
	fscanf(foo, "cpu %d %d %d %d", &user, &nice, &kern, &idle);
	fclose(foo);

	total_change = user + nice + kern + idle -
		(old_user + old_nice + old_kern + old_idle);

	move_left_and_draw(buf,
			   (user - old_user) * AREA_H / (total_change),
			   (nice - old_nice) * AREA_H / (total_change),
			   (kern - old_kern) * AREA_H / (total_change),
			   (info->color >> 16) & 0xff,
			   (info->color >> 8) & 0xff, info->color & 0xff,
			   AREA_X, AREA_W);

	gtk_image_set_from_pixbuf(image, buf);

	old_user = user;
	old_nice = nice;
	old_kern = kern;
	old_idle = idle;

	return TRUE;
}

static void my_gconf_handler(GConfClient * gcc,
			     guint conn, GConfEntry * entry, gpointer data)
{
	PluginInfo *info = (PluginInfo *)data;

	info->color = gconf_value_get_int(gconf_entry_get_value(entry)
		);
}

static void cpu_inact(osso_hw_state_t * state, gpointer user_data)
{
	PluginInfo *info = (PluginInfo *)user_data;

	if (state->system_inactivity_ind) {
		if (info->notify_id) {
			g_source_remove(info->notify_id);
			info->notify_id = 0;
		}
	} else {
		if (!info->notify_id) {
			info->notify_id =
				g_timeout_add(2000, my_timeout, user_data);
		}
	}
}

static void cpu_popup(GtkToggleButton *button, gpointer user_data)
{
	GtkWidget *menu_item;
	GtkWidget *menu;
	PluginInfo *info = (PluginInfo *)user_data;

	if (!gtk_toggle_button_get_active(button))
		return;

	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label(_("Show processes"));
	gtk_widget_show_all(menu_item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	g_signal_connect(menu_item, "activate", G_CALLBACK(top_dialog), NULL);

	menu_item = gtk_separator_menu_item_new();
	gtk_widget_show_all(menu_item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);

	append_items(GTK_MENU_SHELL(menu), info->gcc);

	menu_item = gtk_menu_item_new_with_label("Run command...");
	g_signal_connect(menu_item, "activate", G_CALLBACK(run_cmd_dialog),
			NULL);
	gtk_widget_show_all(menu_item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);

	g_object_weak_ref(G_OBJECT(menu), my_notify, info->button);

	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, popup_place, info->button, 1,
			gtk_get_current_event_time());

	g_signal_connect(menu,
			"selection-done",
			G_CALLBACK(gtk_object_destroy), NULL);

	gtk_widget_show_all(menu);
}

static void my_notify(gpointer data, GObject *has_been)
{
	(void)has_been;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data), FALSE);
}

==> osso-statusbar-cpu_0.5.7.tar.gz: osso-statusbar-cpu-0.5.7/src/osso-statusbar-cpu.c <==
#include <hildon-status-bar-lib/hildon-status-bar-item.h>
#include <stdlib.h>
#include <signal.h>

/* GTK includes */
#include <gtk/gtkicontheme.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtkmenu.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkseparatormenuitem.h>
#include <hildon-widgets/hildon-note.h>

#include <glib.h>

#include "dialog.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include "osso-statusbar-cpu.h"
#include "common.h"

static gboolean my_timeout(gpointer data);
static void my_gconf_handler(GConfClient * gcc,
			     guint conn,
			     GConfEntry * entry, gpointer user_data);
static void my_gconf_clk_handler(GConfClient * gcc,
			         guint conn,
			         GConfEntry * entry, gpointer user_data);
static void cpu_inact(osso_hw_state_t * state, gpointer user_data);
static void cpu_popup(GtkToggleButton *button, gpointer user_data);
static void my_notify(gpointer data, GObject *has_been);

void cpu_entry(HildonStatusBarPluginFn_st *fn)
{
	g_return_if_fail(fn);
	fn->initialize = cpu_initialize;
	fn->destroy = cpu_destroy;
	fn->update = cpu_update;
	fn->get_priority = cpu_get_priority;
}

void *cpu_initialize(HildonStatusBarItem *item, GtkWidget **button)
{
	osso_hw_state_t osso_states = {
		.shutdown_ind = FALSE,
		.save_unsaved_data_ind = FALSE,
		.memory_low_ind = FALSE,
		.system_inactivity_ind = TRUE,
		.sig_device_mode_ind = 0
       	};
	GdkPixbuf *icon_pixbuf = NULL;
	GError *error = NULL;
	PluginInfo *info = NULL;
	GtkIconTheme *theme = NULL;
	g_return_val_if_fail(item, NULL);

	theme = gtk_icon_theme_get_default();
	icon_pixbuf = gtk_icon_theme_load_icon(theme, HILDON_STATUS_BAR_CPU_ICON_NAME,
			40, GTK_ICON_LOOKUP_NO_SVG, NULL);
	if (icon_pixbuf == NULL) {
		gtk_widget_destroy(GTK_WIDGET(item));
		return NULL;
	}

	info = g_new0(PluginInfo, 1);
	g_return_val_if_fail(info, NULL);

	info->item = item;

	info->osso =
		osso_initialize("osso_statusbar_cpu", VERSION, FALSE,
				NULL);
	osso_hw_set_event_cb(info->osso, &osso_states, cpu_inact,
			     (gpointer)info);

	info->gcc = gconf_client_get_default();

	info->button = gtk_toggle_button_new();

	if (error) {
		g_error_free(error);
		error = NULL;
		info->icon = NULL;
	}
	info->icon = gtk_image_new_from_pixbuf(icon_pixbuf);

	info->color = gc_get_int_fall(info->gcc,
				      GCONF_PATH_CPU_COLOR, CPU_DEFAULT);
	info->clock = gconf_client_get_bool(info->gcc,
				            GCONF_PATH_CLOCK_ON, NULL);

	/* icon_pixbuf is owned by info->icon */
	g_object_unref(icon_pixbuf);
	
	gtk_container_add(GTK_CONTAINER(info->button),
			  GTK_WIDGET(info->icon));

	*button = info->button;

	info->notify_id = g_timeout_add(2000, my_timeout, info);

	gconf_client_add_dir(info->gcc,
			     GCONF_PATH, GCONF_CLIENT_PRELOAD_NONE, NULL);

	info->gc_notify_id = gconf_client_notify_add(info->gcc,
						     GCONF_PATH_CPU_COLOR,
						     my_gconf_handler,
						     info, NULL, NULL);
	info->gc_notify_id_clk = gconf_client_notify_add(info->gcc,
						     GCONF_PATH_CLOCK_ON,
						     my_gconf_clk_handler,
						     info, NULL, NULL);

	g_signal_connect(info->button, "toggled", G_CALLBACK(cpu_popup),
			 info);

	if (info->clock) {
		info->clock_id = 
			g_signal_connect_after(info->icon, "expose-event",
					G_CALLBACK(clock_overlay),
					NULL);
	}

	gconf_client_clear_cache(info->gcc);
	if (!gconf_client_get_bool(info->gcc, GCONF_PATH_CPU_ON, NULL)) {
		gtk_object_destroy(GTK_OBJECT(item));
	} else {
		gtk_widget_show_all(info->button);
	}

	return info;
}

void cpu_destroy(void *data)
{
	osso_hw_state_t osso_states = { FALSE, FALSE, FALSE, FALSE, 0 };
	PluginInfo *info;
	g_return_if_fail(data);

	info = (PluginInfo *)data;

	g_source_remove(info->notify_id);
	info->notify_id = 0;

	gconf_client_notify_remove(info->gcc, info->gc_notify_id);
	gconf_client_notify_remove(info->gcc, info->gc_notify_id_clk);
	gconf_client_remove_dir(info->gcc, GCONF_PATH, NULL);

	g_object_unref(G_OBJECT(info->gcc));

	osso_hw_unset_event_cb(info->osso, &osso_states);
	osso_deinitialize(info->osso);

	g_free(data);
}

gint cpu_get_priority(void *data)
{
	return HILDON_STATUS_BAR_CPU_PRIORITY;
}

void cpu_update(void *data, gint value1, gint value2, const gchar *str)
{
	PluginInfo *info = (PluginInfo *)data;
	g_return_if_fail(info);

	if (value2 == 0)
		gtk_widget_destroy(GTK_WIDGET(info->item));
	return;
}

static gboolean my_timeout(gpointer data)
{
	PluginInfo *info = (PluginInfo *)data;
	GtkImage *image = GTK_IMAGE(info->icon);
	GdkPixbuf *buf = gtk_image_get_pixbuf(image);

	FILE *foo = fopen("/proc/stat", "r");
	int user, nice, kern, idle, total_change;
	static int old_user, old_nice, old_kern, old_idle;
	fscanf(foo, "cpu %d %d %d %d", &user, &nice, &kern, &idle);
	fclose(foo);

	total_change = user + nice + kern + idle -
		(old_user + old_nice + old_kern + old_idle);

	move_left_and_draw(buf,
			   (user - old_user) * AREA_H / (total_change),
			   (nice - old_nice) * AREA_H / (total_change),
			   (kern - old_kern) * AREA_H / (total_change),
			   (info->color >> 16) & 0xff,
			   (info->color >> 8) & 0xff, info->color & 0xff,
			   AREA_X, AREA_W);

	gtk_image_set_from_pixbuf(image, buf);

	old_user = user;
	old_nice = nice;
	old_kern = kern;
	old_idle = idle;

	return TRUE;
}

static void my_gconf_handler(GConfClient * gcc,
			     guint conn, GConfEntry * entry, gpointer data)
{
	PluginInfo *info = (PluginInfo *)data;

	info->color = gconf_value_get_int(gconf_entry_get_value(entry)
		);
}

static void my_gconf_clk_handler(GConfClient * gcc,
			     guint conn, GConfEntry * entry, gpointer data)
{
	PluginInfo *info = (PluginInfo *)data;

	info->clock = gconf_value_get_bool(gconf_entry_get_value(entry)
		);
	if (!info->clock && info->clock_id) {
		g_signal_handler_disconnect(info->icon, info->clock_id);
		info->clock_id = 0;
	} else {
		info->clock_id = 
			g_signal_connect_after(info->icon, "expose-event",
				               G_CALLBACK(clock_overlay),
				               NULL);
	}
}

static void cpu_inact(osso_hw_state_t * state, gpointer user_data)
{
	PluginInfo *info = (PluginInfo *)user_data;

	if (state->system_inactivity_ind) {
		if (info->notify_id) {
			g_source_remove(info->notify_id);
			info->notify_id = 0;
		}
	} else {
		if (!info->notify_id) {
			info->notify_id =
				g_timeout_add(2000, my_timeout, user_data);
		}
	}
}

static void cpu_popup(GtkToggleButton *button, gpointer user_data)
{
	GtkWidget *menu_item;
	GtkWidget *menu;
	PluginInfo *info = (PluginInfo *)user_data;

	if (!gtk_toggle_button_get_active(button))
		return;

	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label(_("Show processes"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
	g_signal_connect(menu_item, "activate", G_CALLBACK(top_dialog), NULL);

	menu_item = gtk_separator_menu_item_new();
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);

	append_items(GTK_MENU_SHELL(menu), info->gcc, info->osso);

	g_object_weak_ref(G_OBJECT(menu), my_notify, info->button);

	gtk_widget_show_all(menu);
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, popup_place, info->button, 1,
			gtk_get_current_event_time());

	g_signal_connect(menu,
			"selection-done",
			G_CALLBACK(gtk_object_destroy), NULL);
}

static void my_notify(gpointer data, GObject *has_been)
{
	(void)has_been;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data), FALSE);
}
